const GdkPixbuf = imports.gi.GdkPixbuf;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const GObject = imports.gi.GObject;
const Gtk = imports.gi.Gtk;
const St = imports.gi.St;

const Gettext = imports.gettext.domain('nls1729-extensions');
const _ = Gettext.gettext;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const _N = function(x) { return x; }
const TXT_INSTS = _N("To change text enter new text then press Apply.");
const ICO_INSTS = _N("To add or change icon press Select then choose icon.");
const RST_INSTS = _N("To reset to default settings press Reset.");
const TITLE = _N("Choose Icon");
const DEFAULT_ICO = ExtensionUtils.getCurrentExtension().path + '/face-smile-3.svg'; // From Tango Project - Public Domain
const ACTION = 0;
const GTK_RESPONSE_CANCEL = -6;
const GTK_RESPONSE_ACCEPT = -3;
const GTK_STOCK_CANCEL = 'gtk-cancel';
const GTK_STOCK_OPEN = 'gtk-open';
const NEW_ICO_KEY = 'activities-config-button-icon-path';
const ORI_TXT_KEY = 'original-activities-button-text';
const NEW_TXT_KEY = 'activities-config-button-text';
const NO_HOTC_KEY = 'activities-config-hot-corner';
const NO_ICON_KEY = 'activities-config-button-no-icon';
const NO_TEXT_KEY = 'activities-config-button-no-text';

function init() {
    Convenience.initTranslations();
}

const ActivitiesConfiguratorSettingsWidget = new GObject.Class({
    Name: 'ActivitiesConfigurator.Prefs.ActivitiesConfiguratorSettingsWidget',
    GTypeName: 'ActivitiesConfiguratorSettingsWidget',
    Extends: Gtk.Grid,

    _init: function(params) {	
        this.parent(params);
        this.margin = this.row_spacing = this.column_spacing = 10;
	this._settings = Convenience.getSettings();
        this.attach(new Gtk.Label({ label: _(TXT_INSTS), wrap: true, xalign: 0.0 }), 1, 0, 2, 1);
        this.attach(new Gtk.Label({ label: _(ICO_INSTS), wrap: true, xalign: 0.0 }), 2, 6, 1, 1);
        this.attach(new Gtk.Label({ label: _(RST_INSTS), wrap: true, xalign: 0.0 }), 2, 14, 1, 1);
        let applyBtn = new Gtk.Button({ label: _("Apply ") });
        this.attach(applyBtn, 1, 2, 1, 1);
        this._entry = new Gtk.Entry({ hexpand: true });
        this.attach(this._entry, 2, 2, 1, 1);
        applyBtn.connect('clicked', setActivitiesText);
        this._iconImage = new Gtk.Image();
        this._iconPath = this._settings.get_string(NEW_ICO_KEY) || DEFAULT_ICO;
        this._loadIcon(this._iconPath);
        this.attach(this._iconImage, 0, 6, 1, 1);
        let iconBtn = new Gtk.Button({ label: _("Select") });
        this.attach(iconBtn, 1, 6, 1, 1);
        iconBtn.connect('clicked', setActivitiesIcon);
        let noText = new Gtk.CheckButton({ label: _("If checked text is not displayed.") });
        this._settings.bind(NO_TEXT_KEY, noText, 'active', Gio.SettingsBindFlags.DEFAULT);
        this.attach(noText, 2, 4, 1, 1);
        noText.connect('toggled', setNoText);
        let noIcon = new Gtk.CheckButton({ label: _("If checked icon is not displayed.") });
        this._settings.bind(NO_ICON_KEY, noIcon, 'active', Gio.SettingsBindFlags.DEFAULT);
        this.attach(noIcon, 2, 8, 1, 1);
        noIcon.connect('toggled', setNoIcon);
        let noHotCorner = new Gtk.CheckButton({ label: _("If checked upper left hot corner is disabled.") });
        this._settings.bind(NO_HOTC_KEY, noHotCorner, 'active', Gio.SettingsBindFlags.DEFAULT);
        this.attach(noHotCorner, 2, 10, 1, 1);
        noHotCorner.connect('toggled', setNoHotCorner);
        let horzSeparator = new Gtk.HSeparator();
        this.attach(horzSeparator, 0, 12, 3, 1);
        let defaultsBtn = new Gtk.Button({ label: _("Reset ") } );
        this.attach(defaultsBtn, 1, 14, 1, 1);
        defaultsBtn.connect('clicked', resetSettings);
    },

    _loadIcon: function(path) {
        let pixbuf = new GdkPixbuf.Pixbuf.new_from_file_at_scale(path, 32, 32, null);
        this._iconImage.set_from_pixbuf(pixbuf);
    }
});

let widget;

function setActivitiesIcon() {
    let dialog = new Gtk.FileChooserDialog({ title: _(TITLE), action: ACTION });
    dialog.add_button(GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
    dialog.add_button(GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT);
    dialog.set_filename(widget._iconPath);
    let filter = new Gtk.FileFilter();
    filter.set_name(_("Images"));
    filter.add_pattern("*.png");
    filter.add_pattern("*.jpg");
    filter.add_pattern("*.gif");
    filter.add_pattern("*.svg");
    filter.add_pattern("*.ico");
    dialog.add_filter(filter);
    let response = dialog.run();
    if(response == GTK_RESPONSE_ACCEPT) {
        let filename = dialog.get_filename()
        if(filename != widget._iconPath) {
            widget._iconPath = filename;
            widget._loadIcon(filename);
            widget._settings.set_string(NEW_ICO_KEY, filename);
        }
    }
    dialog.destroy();
}

function resetSettings() {
    let default_txt = widget._settings.get_string(ORI_TXT_KEY);
    widget._settings.set_string(NEW_TXT_KEY, default_txt);
    widget._settings.set_boolean(NO_TEXT_KEY, false);
    widget._settings.set_string(NEW_ICO_KEY, DEFAULT_ICO);
    widget._iconPath = DEFAULT_ICO;
    widget._loadIcon(widget._iconPath);
    widget._settings.set_boolean(NO_ICON_KEY, false);
    widget._settings.set_boolean(NO_HOTC_KEY, false);
}

function setNoIcon(checkbox) {
    widget._settings.set_boolean(NO_ICON_KEY, checkbox.active);
}

function setActivitiesText() {
    let text = widget._entry.get_text();
    if(text != '') {
        widget._settings.set_string(NEW_TXT_KEY,text);
        widget._entry.set_text('');
    }
}

function setNoText(checkbox) {
    widget._settings.set_boolean(NO_TEXT_KEY, checkbox.active);
}

function setNoHotCorner(checkbox) {
    widget._settings.set_boolean(NO_HOTC_KEY, checkbox.active);
}

function buildPrefsWidget() {
    widget = new ActivitiesConfiguratorSettingsWidget();
    widget.show_all();
    return widget;
}
